unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    RedetectButton: TButton;
    DetectLabel: TLabel;
    ResultMemo: TMemo;
    CtEdit: TEdit;
    BenchButton: TButton;
    HighPriority: TCheckBox;
    cbProcessMessages: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure RedetectButtonClick(Sender: TObject);
    procedure BenchButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB;

var
  TicksPerSec: Int64;
  SecsPerTick: Double;
  DeviceIndex: LongWord = diNone;

function MyFloatToStr(F: Double): AnsiString;
begin
  Result := Format('%f', [ F ]);
end;



procedure TForm1.FormCreate(Sender: TObject);
begin
  QueryPerformanceFrequency(TicksPerSec);
  SecsPerTick := 1/TicksPerSec;
  
  RedetectButton.Click;
end;

procedure TForm1.RedetectButtonClick(Sender: TObject);
var
  I: Integer;
  Mask, Status, PID: LongWord;

begin
  for I := 0 to 31 do AIOUSB_CloseDevice(I);
  DeviceIndex := diNone;

  Mask := GetDevices;
  for I := 0 to 31 do if (Mask and (1 shl I)) <> 0 then begin
    Status := QueryDeviceInfo(I, @PID, nil, nil, nil, nil);
    if Status <> ERROR_SUCCESS then Continue;
    if PID < $8140 then Continue;
    if PID > $815D then Continue;

    DeviceIndex := I;
    Break;
  end;

  if DeviceIndex = diNone then
    DetectLabel.Caption := 'Not Found'
  else
    DetectLabel.Caption := 'Found @ ' + IntToStr(DeviceIndex)
  ;
end;

procedure TForm1.BenchButtonClick(Sender: TObject);
var
  I, Ct: Integer;
  DACData: array of Word;
  DIOData: array of Byte;
  Den, Naui: Int64;
  startTime, endTime: TDateTime;
  dT, maxTime, minTime, sumTime, tempTime: Double;
  Status: LongWord;
  timings:array of array[0..1] of Int64;
begin
  if DeviceIndex = diNone then begin
    Beep;
    Exit;
  end;

  Ct := StrToInt(CtEdit.Text);
  SetLength(timings, Ct);

  startTime := Now();
  ResultMemo.Lines.Add('Starting DACDIO_WriteAll() benchmark run of ' + IntToStr(Ct) + ' passes at ' + TimeToStr(startTime));


  BenchButton.Enabled := False;
  SetLength(DACData, 2);
  SetLength(DIOData, 2);
  maxTime := 0.0;
  minTime := 9999.0;
  Status := ERROR_SUCCESS;

  if HighPriority.checked then begin
    SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
    ResultMemo.Lines.Add('Process Priority set to HIGH_PRIORITY_CLASS');
  end
  else begin
    SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
    ResultMemo.Lines.Add('Process Priority set to NORMAL_PRIORITY_CLASS');
  end;

  if cbProcessMessages.Checked then begin
    ResultMemo.Lines.Add('Message Queue PROCESSED in loop.');
    QueryPerformanceCounter(Den);
    for I := 0 to Ct-1 do begin
      QueryPerformanceCounter( timings[I][0] );
      Status := DACDIO_WriteAll(DeviceIndex, @DACData[0], @DIOData[0]);
      QueryPerformanceCounter( timings[I][1] );
      Application.ProcessMessages;
    end;
    QueryPerformanceCounter(Naui);
  end else begin
    ResultMemo.Lines.Add('Message Queue IGNORED in loop.');
    QueryPerformanceCounter(Den);
    for I := 0 to Ct-1 do begin
      QueryPerformanceCounter( timings[I][0] );
      Status := DACDIO_WriteAll(DeviceIndex, @DACData[0], @DIOData[0]);
      QueryPerformanceCounter( timings[I][1] );
    end;
    QueryPerformanceCounter(Naui);
  end;
  SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);

  if Status <> ERROR_SUCCESS then begin
    ResultMemo.Lines.Add('Error ' + IntToStr(Status));
  end;

  dT := 1000000 * SecsPerTick * (Naui - Den);
  sumTime := 0.0;
  ResultMemo.Lines.Add(IntToStr(Ct) + ' in ' + MyFloatToStr(dT) + 's, ' + MyFloatToStr(dT/Ct) + 's each' );
  for I := 0 to Ct-1 do begin
    tempTime := 1000000 * SecsPerTick * (timings[I][1] - timings[I][0]);
    sumTime := sumTime + dT;
    if (tempTime) > maxtime then maxtime := tempTime;
    if (tempTime) < mintime then mintime := tempTime;
  end;
  ResultMemo.Lines.Add('Shortest call took ' + MyFloatToStr(minTime) + 's, Longest took ' + MyFloatToStr(maxtime) + 's.' );

  endTime := Now();
  ResultMemo.Lines.Add('Ended at ' + TimeToStr(endTime) );
  ResultMemo.Lines.Add('');
  BenchButton.Enabled := True;
end;

end.
